@@ -44,22 +44,22 @@ def test_format(config, workspace):
     res = pylsp_format_document(config, workspace, doc, options=None)
 
     assert len(res) == 1
-    assert res[0]['newText'] == "a = 123\n\n\ndef func():\n    pass\n"
+    assert res[0]["newText"] == "a = 123\n\n\ndef func():\n    pass\n"
 
 
 def test_range_format(config, workspace):
     doc = Document(DOC_URI, workspace, DOC)
 
     def_range = {
-        'start': {'line': 0, 'character': 0},
-        'end': {'line': 2, 'character': 0}
+        "start": {"line": 0, "character": 0},
+        "end": {"line": 2, "character": 0},
     }
     res = pylsp_format_range(config, workspace, doc, def_range, options=None)
 
     assert len(res) == 1
 
     # Make sure the func is still badly formatted
-    assert res[0]['newText'] == "a = 123\n\n\n\n\ndef func():\n    pass\n"
+    assert res[0]["newText"] == "a = 123\n\n\n\n\ndef func():\n    pass\n"
 
 
 def test_no_change(config, workspace):
@@ -72,12 +72,15 @@ def test_hanging_indentation(config, workspace):
     res = pylsp_format_document(config, workspace, doc, options=None)
 
     assert len(res) == 1
-    assert res[0]['newText'] == CORRECT_INDENTED_DOC
+    assert res[0]["newText"] == CORRECT_INDENTED_DOC
 
 
-@pytest.mark.parametrize('newline', ['\r\n', '\r'])
+@pytest.mark.parametrize("newline", ["\r\n", "\r"])
 def test_line_endings(config, workspace, newline):
-    doc = Document(DOC_URI, workspace, f'import os;import sys{2 * newline}dict(a=1)')
+    doc = Document(DOC_URI, workspace, f"import os;import sys{2 * newline}dict(a=1)")
     res = pylsp_format_document(config, workspace, doc, options=None)
 
-    assert res[0]['newText'] == f'import os{newline}import sys{2 * newline}dict(a=1){newline}'
+    assert (
+        res[0]["newText"]
+        == f"import os{newline}import sys{2 * newline}dict(a=1){newline}"
+    )
